@@ -1,7 +1,7 @@
 use anyhow::Result;
 use std::future::Future;
+use std::sync::Arc;
 use std::time::Duration;
-use std::{panic, sync::Arc};
 
 use crate::exit_state::prettify_exit_states;
 use crate::shutdown_token::create_shutdown_token;
@@ -43,33 +43,14 @@ pub struct Toplevel {
     subsys_handle: SubsystemHandle,
 }
 
-impl Drop for Toplevel {
-    fn drop(&mut self) {
-        // Restore panic hook to its original state
-        let _ = panic::take_hook();
-    }
-}
-
 impl Toplevel {
     /// Creates a new Toplevel object.
     ///
     /// The Toplevel object is the base for everything else in this crate.
-    ///
-    /// During creation, a panic hook is registered to cause a graceful system
-    /// shutdown in case a panic happens.
-    /// This prevents a program hang that might happen when multithreaded asynchronous
-    /// programs experience a panic on one thread.
     #[allow(clippy::new_without_default)]
     pub fn new() -> Self {
         let shutdown_token = create_shutdown_token();
 
-        // Register panic handler to trigger shutdown token
-        let panic_shutdown_token = shutdown_token.clone();
-        panic::set_hook(Box::new(move |panic_info| {
-            log::error!("ERROR: {}", panic_info);
-            panic_shutdown_token.shutdown();
-        }));
-
         let subsys_data = Arc::new(SubsystemData::new("", shutdown_token));
         let subsys_handle = SubsystemHandle::new(subsys_data.clone());
         Self {
@@ -102,14 +83,13 @@ impl Toplevel {
     /// * `subsystem` - The subsystem to be started
     ///
     pub fn start<
-        Fut: Future<Output = Result<()>> + Send,
+        Fut: 'static + Future<Output = Result<()>> + Send,
         S: 'static + FnOnce(SubsystemHandle) -> Fut + Send,
     >(
         self,
         name: &'static str,
         subsystem: S,
     ) -> Self {
-        //self.subsys_data.start(name, subsystem);
         SubsystemHandle::new(self.subsys_data.clone()).start(name, subsystem);
 
         self
